/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: array_geometry.h,v 1.4 2002/05/08 21:24:47 jdejong Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jdejong 2002-05-08  imported
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */
/* $Id: array_geometry.h,v 1.4 2002/05/08 21:24:47 jdejong Exp $ */

#ifndef _ARRAY_GEOMETRY_TABLE_H
#define _ARRAY_GEOMETRY_TABLE_H
/*----------------------------------------------------------------------- */
/* Array Geometry table FITS binary tables routines                       */
/*----------------------------------------------------------------------- */
#include "VLTItable.h"

typedef struct      /* table definition for array geometry table */
{
  table_def *def;       /* basic table information */
  long   row;           /* last row (0-rel) read or written, -1=> none. */
  char*  arrnam;        /* Name of the array */
  char   *date_obs;     /* observing date */
  char*  frame;         /* Coordinate frame, e.g. 'GEOCENTRIC' */
  double arrayx;        /* X coordinate of array center */
  double arrayy;        /* Y coordinate of array center */
  double arrayz;        /* Z coordinate of array center */
  int    sta_name_col;  /* Station name column number (0-rel) */
  int    tel_name_col;  /* Telescope name column number (0-rel) */
  int    index_col;     /* Station number column number */
  int    diameter_col;  /* Diameter column number */
  int    staxyz_col;    /* Station location column number */
  int    mntsta_col;    /* mount type column number */
  int    mntparam_col;  /* mount parameters column number */
} array_geometry_table_def;

typedef struct      /* structure used for array_geometry_table row */
{
  double staxyz[3];  /* Station coordinates (m) */
  float  diameter;   /* Station aperature size (m) */
  int    index;      /* Station number */
  int    mntsta;     /* Station mount type code */
  char   sta_name[10]; /* Station name (8 char) */
  char   tel_name[10]; /* Telescope name (8 char) */
  double *mntparam;  /* Station mount parameter array */
  table_row *raw;    /* generic table row */
} array_geometry_table_row;

/* public functions */

/* array_geometry_table_row functions */
array_geometry_table_row* make_array_geometry_table_row(array_geometry_table_def* table);
/*----------------------------------------------------------------------- */
/*   Make an array_geometry_table_row structure                           */
/*   Inputs:                                                              */
/*      table     pointer to array_geometry_table table structure         */
/*   Returns:                                                             */
/*      An array_geometry_table_row structure pointer ; NULL => failure   */
/*      Data pointers set to generic structure values                     */
/*----------------------------------------------------------------------- */

void kill_array_geometry_table_row (array_geometry_table_row *row);
/*----------------------------------------------------------------------- */
/*   Delete an array_geometry_table_row structure                         */
/*   Inputs:                                                              */
/*      row    pointer to the array_geometry_table_row structure.         */
/*----------------------------------------------------------------------- */

/* array_geometry_table functions */

array_geometry_table_def* 
make_array_geometry_table_def(fitsfile *fptr, char *arrnam,  char *date_obs,
			      double arrayxyz[3], int *ierr);
/*----------------------------------------------------------------------- */
/*   Make a new array_geometry_table in an open FITS file                 */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      arrnam    Array name                                              */
/*      date_obs  Date (yyyy-mm-dd) observations began                    */
/*      arrayxyz  Geodetic position of array center (m)                   */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      A array_geometry_table_def pointer; NULL indicates failure        */
/*----------------------------------------------------------------------- */

array_geometry_table_def* 
open_array_geometry_table(fitsfile *fptr, int *ierr);
/*----------------------------------------------------------------------- */
/*   Open an existing array_geometry_table in the active FITS file        */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      An array_geometry_table_def pointer; NULL indicates failure       */
/*----------------------------------------------------------------------- */

int read_array_geometry_table(fitsfile *fptr,  array_geometry_table_def* table, 
			    array_geometry_table_row *row, long row_number,
			    int *ierr);
/*----------------------------------------------------------------------- */
/*   Read the next row from an array_geometry_table                       */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      table     table definitions structure from open or create         */
/*      row_number which row (1-rel) in the table                         */   
/*                 -1 => next row                                         */  
/*   Input/output:                                                        */
/*      row       pointer to an extant row structure                      */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

int write_array_geometry_table(fitsfile *fptr, array_geometry_table_def *table, 
			     array_geometry_table_row *row,  long row_number,
			     int *ierr);
/*----------------------------------------------------------------------- */
/*   Write  the next row to an array_geometry_table                               */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      table     table definitions structure from open or create         */
/*      row       pointer to an extant row structure                      */
/*      row_number which row (1-rel) in the table                         */
/*                 -1 => next row                                         */  
/*   Input/output:                                                        */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

int close_array_geometry_table(fitsfile *fptr, array_geometry_table_def* table, 
			     int *ierr);
/*----------------------------------------------------------------------- */
/*   Close an open array_geometry_table, associated structures are        */
/*   destroyed                                                            */
/*   Note: this closes the full FITS file                                 */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

void kill_array_geometry_table_def (array_geometry_table_def* table);
/*----------------------------------------------------------------------- */
/*   Delete an array_geometry_table structure                             */
/*   Inputs:                                                              */
/*      table     table definitions structure from open or create         */
/*----------------------------------------------------------------------- */

#endif /* end ARRAY_GEOMETRY_TABLE_H */
/* */
